@@ -470,7 +470,7 @@ cdk_utf8_decode (const char * string, size_t length, int delim)
 }
 
 
-#ifdef __MINGW32__
+#ifndef HAVE_VASPRINTF
 /* 
  * Like vsprintf but provides a pointer to malloc'd storage, which
  * must be freed by the caller (gcry_free).  Taken from libiberty as
