@@ -1,5 +1,5 @@
 /* opencdk.h - Open Crypto Development Kit (OpenCDK)
- *        Copyright (C) 2001, 2002, 2003, 2005 Timo Schulz
+ *        Copyright (C) 2001, 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
  *
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with OpenCDK; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
+ * Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #ifndef OPENCDK_H
@@ -23,7 +23,7 @@
 
 #include <stdarg.h>
 
-#define OPENCDK_VERSION "0.5.7"
+#define OPENCDK_VERSION "0.5.3"
 
 #ifdef __cplusplus
 extern "C" {
@@ -266,8 +266,7 @@ enum cdk_key_flag_t {
     CDK_KEY_VALID   = 0,
     CDK_KEY_INVALID = 1, /* missing or wrong self signature */
     CDK_KEY_EXPIRED = 2,
-    CDK_KEY_REVOKED = 4,
-    CDK_KEY_NOSIGNER= 8,
+    CDK_KEY_REVOKED = 4
 };
 
 enum cdk_trust_flag_t {
@@ -533,7 +532,7 @@ typedef struct cdk_packet_s *cdk_packet_t;
 /* memory routines */
 typedef void (*cdk_log_fnc_t) (void *, int, const char *, va_list);
 void cdk_set_log_level (int lvl);
-void cdk_set_log_handler (cdk_log_fnc_t logfnc, void * opaq);
+void cdk_set_log_handler (cdk_log_fnc_t logfnc, void * opaque);
 const char* cdk_strerror (int ec);
 void cdk_set_malloc_hooks (void *(*new_alloc_func) (size_t n),
                            void *(*new_alloc_secure_func) (size_t n),
