@@ -1,5 +1,5 @@
-/* -*- Mode: C; c-file-style: "bsd" -*- */
-/* pubkey.c - Public key API
+/* -*- Mode: C; c-file-style: "bsd" -*-
+ * pubkey.c - Public key API
  *        Copyright (C) 2002, 2003 Timo Schulz
  *
  * This file is part of OpenCDK.
@@ -27,12 +27,13 @@
 #include "opencdk.h"
 #include "main.h"
 #include "packet.h"
+#include "cipher.h"
 
 
-static GCRY_MPI *
-convert_to_gcrympi( CDK_MPI m[4], int ncount )
+static gcry_mpi_t *
+convert_to_gcrympi( cdk_mpi_t m[4], int ncount )
 {
-    GCRY_MPI * d;
+    gcry_mpi_t * d;
     size_t nbytes = 0;
     int i = 0, rc = 0;
 
@@ -43,13 +44,13 @@ convert_to_gcrympi( CDK_MPI m[4], int ncount )
         return NULL;
     for( i = 0; i < ncount; i++ ) {
         nbytes = m[i]->bytes + 2;
-        if( gcry_mpi_scan( &d[i], GCRYMPI_FMT_PGP, m[i]->data, &nbytes ) ) {
+        if( gcry_mpi_scan( &d[i], GCRYMPI_FMT_PGP, m[i]->data, nbytes, &nbytes ) ) {
             rc = CDK_Gcry_Error;
             break;
 	}
     }
     if( rc ) {
-        _cdk_free_mpi_array( i, d );
+        _cdk_free_mpibuf( i, d );
         d = NULL;
     }
     return d;
@@ -57,11 +58,11 @@ convert_to_gcrympi( CDK_MPI m[4], int ncount )
 
 
 static int
-seckey_to_sexp( GCRY_SEXP * r_skey, cdkPKT_secret_key * sk )
+seckey_to_sexp( gcry_sexp_t * r_skey, cdk_pkt_seckey_t sk )
 {
-    GCRY_SEXP sexp = NULL;
-    GCRY_MPI * mpk = NULL, * msk = NULL;
-    cdkPKT_public_key * pk = NULL;
+    gcry_sexp_t sexp = NULL;
+    gcry_mpi_t * mpk = NULL, * msk = NULL;
+    cdk_pkt_pubkey_t pk;
     const char * fmt = NULL;
     int ncount = 0, nscount = 0;
     int rc = 0;
@@ -99,18 +100,18 @@ seckey_to_sexp( GCRY_SEXP * r_skey, cdkPKT_secret_key * sk )
     else
         rc = CDK_Inv_Algo;
 
-    _cdk_free_mpi_array( ncount, mpk );
-    _cdk_free_mpi_array( nscount, msk );
+    _cdk_free_mpibuf( ncount, mpk );
+    _cdk_free_mpibuf( nscount, msk );
     *r_skey = sexp;
     return rc;
 }
 
 
 static int
-pubkey_to_sexp( GCRY_SEXP * r_key, cdkPKT_public_key * pk )
+pubkey_to_sexp( gcry_sexp_t * r_key, cdk_pkt_pubkey_t pk )
 {
-    GCRY_SEXP sexp = NULL;
-    GCRY_MPI * m;
+    gcry_sexp_t sexp = NULL;
+    gcry_mpi_t * m;
     const char * fmt = NULL;
     int ncount = 0;
     int rc = 0;
@@ -139,16 +140,16 @@ pubkey_to_sexp( GCRY_SEXP * r_key, cdkPKT_public_key * pk )
     }
     else
         rc = CDK_Inv_Algo;
-    _cdk_free_mpi_array( ncount, m );
+    _cdk_free_mpibuf( ncount, m );
     *r_key = sexp;
     return rc;
 }
 
 
 static int
-enckey_to_sexp( GCRY_SEXP * r_sexp, GCRY_MPI esk )
+enckey_to_sexp( gcry_sexp_t * r_sexp, gcry_mpi_t esk )
 {
-    GCRY_SEXP sexp = NULL;
+    gcry_sexp_t sexp = NULL;
     int rc = 0;
 
     if( !r_sexp || !esk )
@@ -161,19 +162,19 @@ enckey_to_sexp( GCRY_SEXP * r_sexp, GCRY_MPI esk )
 
 
 static int
-digest_to_sexp( GCRY_SEXP * r_md, int algo, const byte * md, size_t mdlen )
+digest_to_sexp( gcry_sexp_t * r_md, int algo, const byte * md, size_t mdlen )
 {
-    GCRY_SEXP sexp = NULL;
-    GCRY_MPI m = NULL;
+    gcry_sexp_t sexp = NULL;
+    gcry_mpi_t m = NULL;
     size_t nbytes = 0;
     int rc = 0;
 
     if( !r_md || !md )
         return CDK_Inv_Value;
-    nbytes = mdlen ? mdlen : gcry_md_get_algo_dlen( algo );
+    nbytes = mdlen ? mdlen : cdk_md_get_algo_dlen( algo );
     if( !nbytes )
         return CDK_Inv_Algo;
-    if( gcry_mpi_scan( &m, GCRYMPI_FMT_USG, md, &nbytes ) )
+    if( gcry_mpi_scan( &m, GCRYMPI_FMT_USG, md, nbytes, &nbytes ) )
         return CDK_Gcry_Error;
     if( gcry_sexp_build( &sexp, NULL, "%m", m ) )
         rc = CDK_Gcry_Error;
@@ -185,11 +186,11 @@ digest_to_sexp( GCRY_SEXP * r_md, int algo, const byte * md, size_t mdlen )
 
 
 static int
-sexp_to_bitmpi( GCRY_SEXP sexp, const char * val, CDK_MPI * ret_buf )
+sexp_to_bitmpi( gcry_sexp_t sexp, const char * val, cdk_mpi_t * ret_buf )
 {
-    GCRY_SEXP list = NULL;
-    GCRY_MPI m = NULL;
-    CDK_MPI buf = NULL;
+    gcry_sexp_t list = NULL;
+    gcry_mpi_t m = NULL;
+    cdk_mpi_t buf = NULL;
     size_t nbits = 0, nbytes = 0;
     int rc = 0;
 
@@ -213,10 +214,10 @@ sexp_to_bitmpi( GCRY_SEXP sexp, const char * val, CDK_MPI * ret_buf )
     }
     buf->data[0] = nbits >> 8;
     buf->data[1] = nbits;
-    if( gcry_mpi_print( GCRYMPI_FMT_USG, NULL, &nbytes, m ) )
+    if( gcry_mpi_print( GCRYMPI_FMT_USG, NULL, nbytes, &nbytes, m ) )
         rc = CDK_Gcry_Error;
     else
-        if( gcry_mpi_print( GCRYMPI_FMT_USG, buf->data + 2, &nbytes, m ) )
+        if( gcry_mpi_print( GCRYMPI_FMT_USG, buf->data + 2, nbytes, &nbytes, m ) )
             rc = CDK_Gcry_Error;
     if( !rc ) {
         buf->bytes = nbytes;
@@ -232,7 +233,7 @@ sexp_to_bitmpi( GCRY_SEXP sexp, const char * val, CDK_MPI * ret_buf )
 
 
 static int
-sexp_to_sig( cdkPKT_signature * sig, GCRY_SEXP sexp)
+sexp_to_sig( cdk_pkt_signature_t sig, gcry_sexp_t sexp)
 {
     int rc = 0;
 
@@ -252,10 +253,10 @@ sexp_to_sig( cdkPKT_signature * sig, GCRY_SEXP sexp)
 
 
 static int
-sig_to_sexp( GCRY_SEXP * r_sig, cdkPKT_signature * sig )
+sig_to_sexp( gcry_sexp_t * r_sig, cdk_pkt_signature_t sig )
 {
-    GCRY_SEXP sexp = NULL;
-    GCRY_MPI * m;
+    gcry_sexp_t sexp = NULL;
+    gcry_mpi_t * m;
     const char * fmt;
     int ncount = 0;
     int rc = 0;
@@ -284,14 +285,14 @@ sig_to_sexp( GCRY_SEXP * r_sig, cdkPKT_signature * sig )
     }
     else
         rc = CDK_Inv_Algo;
-    _cdk_free_mpi_array( ncount, m );
+    _cdk_free_mpibuf( ncount, m );
     *r_sig = sexp;
     return rc;
 }
 
 
 static int
-sexp_to_pubenc( cdkPKT_pubkey_enc * enc, GCRY_SEXP sexp)
+sexp_to_pubenc( cdk_pkt_pubkey_enc_t enc, gcry_sexp_t sexp)
 {
     int rc;
 
@@ -311,10 +312,10 @@ sexp_to_pubenc( cdkPKT_pubkey_enc * enc, GCRY_SEXP sexp)
 
 
 static int
-pubenc_to_sexp( GCRY_SEXP * r_sexp, cdkPKT_pubkey_enc * enc)
+pubenc_to_sexp( gcry_sexp_t * r_sexp, cdk_pkt_pubkey_enc_t enc)
 {
-    GCRY_SEXP sexp = NULL;
-    GCRY_MPI * m;
+    gcry_sexp_t sexp = NULL;
+    gcry_mpi_t * m;
     const char * fmt;
     int ncount;
     int rc = 0;
@@ -338,14 +339,14 @@ pubenc_to_sexp( GCRY_SEXP * r_sexp, cdkPKT_pubkey_enc * enc)
     }
     else
         rc = CDK_Inv_Algo;
-    _cdk_free_mpi_array( ncount, m );
+    _cdk_free_mpibuf( ncount, m );
     *r_sexp = sexp;
     return rc;
 }
 
 
 static int
-is_unprotected( cdkPKT_secret_key * sk )
+is_unprotected( cdk_pkt_seckey_t sk )
 {
     if( sk->is_protected && !sk->mpi[0] )
         return 0;
@@ -362,11 +363,11 @@ is_unprotected( cdkPKT_secret_key * sk )
  * Encrypt the session key in @esk and write its encrypted content
  * into the @pke struct.
  **/
-int
-cdk_pk_encrypt( cdkPKT_public_key * pk, cdkPKT_pubkey_enc * pke,
-                CDK_SESSKEY esk )
+cdk_error_t
+cdk_pk_encrypt( cdk_pkt_pubkey_t pk, cdk_pkt_pubkey_enc_t pke,
+                cdk_sesskey_t esk )
 {
-    GCRY_SEXP s_data = NULL, s_pkey = NULL, s_ciph = NULL;
+    gcry_sexp_t s_data = NULL, s_pkey = NULL, s_ciph = NULL;
     int rc;
 
     if( !pk || !esk || !pke )
@@ -398,11 +399,11 @@ cdk_pk_encrypt( cdkPKT_public_key * pk, cdkPKT_pubkey_enc * pke,
  *
  * Decrypt the encrypted session key from @pke into @r_sk.
  **/
-int
-cdk_pk_decrypt( cdkPKT_secret_key * sk, cdkPKT_pubkey_enc * pke,
-                CDK_SESSKEY * r_sk )
+cdk_error_t
+cdk_pk_decrypt( cdk_pkt_seckey_t sk, cdk_pkt_pubkey_enc_t pke,
+                cdk_sesskey_t * r_sk )
 {
-    GCRY_SEXP s_data = NULL, s_skey = NULL, s_plain = NULL;
+    gcry_sexp_t s_data = NULL, s_skey = NULL, s_plain = NULL;
     int rc;
 
     if( !sk || !r_sk || !pke )
@@ -437,10 +438,10 @@ cdk_pk_decrypt( cdkPKT_secret_key * sk, cdkPKT_pubkey_enc * pke,
  *
  * Sign the message digest from @md and write the result into @sig.
  **/
-int
-cdk_pk_sign( cdkPKT_secret_key * sk, cdkPKT_signature * sig, const byte * md )
+cdk_error_t
+cdk_pk_sign( cdk_pkt_seckey_t sk, cdk_pkt_signature_t sig, const byte * md )
 {
-    GCRY_SEXP s_skey = NULL, s_sig = NULL, s_hash = NULL;
+    gcry_sexp_t s_skey = NULL, s_sig = NULL, s_hash = NULL;
     byte * encmd = NULL;
     size_t enclen = 0;
     int nbits, rc;
@@ -484,10 +485,10 @@ cdk_pk_sign( cdkPKT_secret_key * sk, cdkPKT_signature * sig, const byte * md )
  *
  * Verify the signature in @sig and compare it with the message digest in @md.
  **/
-int
-cdk_pk_verify( cdkPKT_public_key * pk, cdkPKT_signature * sig, const byte * md)
+cdk_error_t
+cdk_pk_verify( cdk_pkt_pubkey_t pk, cdk_pkt_signature_t sig, const byte * md)
 {
-    GCRY_SEXP s_pkey = NULL, s_sig = NULL, s_hash = NULL;
+    gcry_sexp_t s_pkey = NULL, s_sig = NULL, s_hash = NULL;
     byte * encmd = NULL;
     size_t enclen = 0;
     int nbits, rc;
@@ -516,7 +517,7 @@ cdk_pk_verify( cdkPKT_public_key * pk, cdkPKT_signature * sig, const byte * md)
 
 
 int
-cdk_pk_get_nbits( cdkPKT_public_key * pk)
+cdk_pk_get_nbits( cdk_pkt_pubkey_t pk )
 {
     if( !pk || !pk->mpi[0] )
         return 0;
@@ -527,33 +528,47 @@ cdk_pk_get_nbits( cdkPKT_public_key * pk)
 int
 cdk_pk_get_npkey( int algo )
 {
-    int n = gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NPKEY, NULL, 0 );
-    return n > 0 ? n : 0;
+size_t bytes;
+
+    if (gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NPKEY, NULL, &bytes))
+    	return 0;
+    
+    return bytes;
 }
 
 
 int
 cdk_pk_get_nskey( int algo )
 {  
-    int n = gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NSKEY, NULL, 0 );
-    n -= cdk_pk_get_npkey( algo );
-    return n > 0 ? n : 0;
+size_t bytes;
+
+    if (gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NSKEY, NULL, &bytes))
+    	return 0;
+    
+    bytes -= cdk_pk_get_npkey( algo );
+    return bytes;
 }
 
 
 int
 cdk_pk_get_nsig( int algo )
 {
-    int n = gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NSIGN, NULL, 0 );
-    return n > 0 ? n : 0;
+size_t bytes;
+
+    if (gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NSIGN, NULL, &bytes))
+    	return 0;
+    return bytes;
 }
 
 
 int
 cdk_pk_get_nenc( int algo )
 {
-    int n = gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NENCR, NULL, 0 );
-    return n > 0 ? n : 0;
+size_t bytes;
+
+    if (gcry_pk_algo_info( algo, GCRYCTL_GET_ALGO_NENCR, NULL, &bytes))
+    	return 0;
+    return bytes;
 }
 
 
@@ -563,12 +578,12 @@ _cdk_pk_algo_usage( int algo )
     int usage = 0;
 
     switch( algo ) {
-    case CDK_PK_RSA  : usage = GCRY_PK_USAGE_SIGN | GCRY_PK_USAGE_ENCR; break;
-    case CDK_PK_RSA_E: usage = GCRY_PK_USAGE_ENCR; break;
-    case CDK_PK_RSA_S: usage = GCRY_PK_USAGE_SIGN; break;
-    case CDK_PK_ELG  : usage = GCRY_PK_USAGE_SIGN | GCRY_PK_USAGE_ENCR; break;
-    case CDK_PK_ELG_E: usage = GCRY_PK_USAGE_ENCR; break;
-    case CDK_PK_DSA  : usage = GCRY_PK_USAGE_SIGN; break; 
+    case CDK_PK_RSA  : usage = PK_USAGE_SIGN | PK_USAGE_ENCR; break;
+    case CDK_PK_RSA_E: usage = PK_USAGE_ENCR; break;
+    case CDK_PK_RSA_S: usage = PK_USAGE_SIGN; break;
+    case CDK_PK_ELG  : usage = PK_USAGE_SIGN | PK_USAGE_ENCR; break;
+    case CDK_PK_ELG_E: usage = PK_USAGE_ENCR; break;
+    case CDK_PK_DSA  : usage = PK_USAGE_SIGN; break; 
     }
     return usage;  
 }
@@ -580,13 +595,13 @@ _cdk_pk_test_algo( int algo, unsigned int usage_flags )
     size_t n = usage_flags;
   
     if( algo < 0 || algo > 110 )
-        return GCRYERR_INV_PK_ALGO;
+        return GPG_ERR_INV_PARAMETER;
     return gcry_pk_algo_info( algo, GCRYCTL_TEST_ALGO, NULL, &n );    
 }
 
 
 static int
-read_mpi( CDK_MPI a, byte * buf, size_t * r_count, size_t * r_nbits )
+read_mpi( cdk_mpi_t a, byte * buf, size_t * r_count, size_t * r_nbits )
 {
     if( !a || !buf || !r_count )
         return CDK_Inv_Value;
@@ -601,8 +616,8 @@ read_mpi( CDK_MPI a, byte * buf, size_t * r_count, size_t * r_nbits )
 }
 
   
-int
-cdk_pk_get_mpi( cdkPKT_public_key * pk, int idx,
+cdk_error_t
+cdk_pk_get_mpi( cdk_pkt_pubkey_t pk, int idx,
                 byte * buf, size_t * r_count, size_t * r_nbits )
 {
     if( !pk || idx < 0 || !r_count )
@@ -613,8 +628,8 @@ cdk_pk_get_mpi( cdkPKT_public_key * pk, int idx,
 }
 
 
-int
-cdk_sk_get_mpi( cdkPKT_secret_key * sk, int idx,
+cdk_error_t
+cdk_sk_get_mpi( cdk_pkt_seckey_t sk, int idx,
                 byte * buf, size_t * r_count, size_t * r_nbits)
 {
     if( !sk || idx < 0 || !r_count)
@@ -626,7 +641,7 @@ cdk_sk_get_mpi( cdkPKT_secret_key * sk, int idx,
 
 
 static u16
-checksum_mpi( CDK_MPI m )
+checksum_mpi( cdk_mpi_t m )
 {
     int i;
     u16 chksum = 0;
@@ -639,34 +654,12 @@ checksum_mpi( CDK_MPI m )
 }
 
 
-static GCRY_CIPHER_HD
-cipher_open( int algo, const byte * key, size_t keylen,
-             const byte * iv, size_t ivlen )
+cdk_error_t
+cdk_sk_unprotect( cdk_pkt_seckey_t sk, const char * pw )
 {
-    GCRY_CIPHER_HD hd;
-    int flags = GCRY_CIPHER_SECURE | GCRY_CIPHER_ENABLE_SYNC;
-    int rc = 0;
-    
-    hd = gcry_cipher_open( algo, GCRY_CIPHER_MODE_CFB, flags );
-    if( hd ) {
-        rc = gcry_cipher_setkey( hd, key, keylen  );
-        if( !rc )
-            rc = gcry_cipher_setiv( hd, iv, ivlen  );
-        if( rc ) {
-            gcry_cipher_close( hd );
-            hd = NULL;
-        }
-    }
-    return hd;
-}
-
-
-int
-cdk_sk_unprotect( cdkPKT_secret_key * sk, const char * pw )
-{
-    GCRY_CIPHER_HD hd;
-    CDK_DEK dek = NULL;
-    CDK_MPI a;
+    cdk_cipher_hd_t hd;
+    cdk_dek_t dek = NULL;
+    cdk_mpi_t a;
     u16 chksum = 0;
     size_t ndata, nbits;
     int j, i, dlen, pos = 0, nskey;
@@ -682,11 +675,12 @@ cdk_sk_unprotect( cdkPKT_secret_key * sk, const char * pw )
                                       sk->protect.s2k, 0, pw );
         if( rc )
             return rc;
-        hd = cipher_open( sk->protect.algo, dek->key, dek->keylen,
-                          sk->protect.iv, sk->protect.ivlen );
+        hd = cdk_cipher_open( sk->protect.algo, 1,
+                              dek->key, dek->keylen,
+                              sk->protect.iv, sk->protect.ivlen );
         if( !hd ) {
             cdk_free( dek );
-            return CDK_Gcry_Error;
+            return CDK_Inv_Algo;
         }
         wipemem( dek, sizeof dek );
         cdk_dek_free( dek );
@@ -696,27 +690,27 @@ cdk_sk_unprotect( cdkPKT_secret_key * sk, const char * pw )
             data = cdk_salloc( ndata, 1 );
             if( !data )
                 return CDK_Out_Of_Core;
-            gcry_cipher_decrypt( hd, data, ndata, sk->encdata, ndata );
+            cdk_cipher_decrypt( hd, data, sk->encdata, ndata );
             if( sk->protect.sha1chk ) {
                 /* This is the new SHA1 checksum method to detect tampering
                    with the key as used by the Klima/Rosa attack */
                 sk->csum = 0;
                 chksum = 1;
-                dlen = gcry_md_get_algo_dlen( GCRY_MD_SHA1 );
+                dlen = cdk_md_get_algo_dlen( CDK_MD_SHA1 );
                 if( ndata < dlen ) {
                     cdk_free( data );
                     return CDK_Inv_Packet;
                 }
                 else {
-                    GCRY_MD_HD md = gcry_md_open( GCRY_MD_SHA1, 1 );
+                    cdk_md_hd_t md = cdk_md_open( CDK_MD_SHA1, 1 );
                     if( !md )
                         return CDK_Gcry_Error;
-                    gcry_md_write( md, data, ndata - dlen );
-                    gcry_md_final( md );
-                    if( !memcmp( gcry_md_read( md, GCRY_MD_SHA1 ),
+                    cdk_md_write( md, data, ndata - dlen );
+                    cdk_md_final( md );
+                    if( !memcmp( cdk_md_read( md, CDK_MD_SHA1 ),
                                  data + ndata - dlen, dlen ) )
                         chksum = 0;	/* digest does match */
-                    gcry_md_close( md );
+                    cdk_md_close( md );
 		}
 	    }
             else {
@@ -746,12 +740,12 @@ cdk_sk_unprotect( cdkPKT_secret_key * sk, const char * pw )
             chksum = 0;
             for( i = 0; i < nskey; i++ ) {
                 a = sk->mpi[i];
-                gcry_cipher_sync( hd );
-                gcry_cipher_decrypt( hd, a->data + 2, a->bytes, NULL, 0 );
+                cdk_cipher_sync( hd );
+                cdk_cipher_decrypt( hd, a->data+2, a->data+2, a->bytes );
                 chksum += checksum_mpi( a );
 	    }
 	}
-        gcry_cipher_close( hd );
+        cdk_cipher_close( hd );
     }
     else {
         chksum = 0;
@@ -765,14 +759,14 @@ cdk_sk_unprotect( cdkPKT_secret_key * sk, const char * pw )
 }
 
 
-int
-cdk_sk_protect( cdkPKT_secret_key * sk, const char * pass )
+cdk_error_t
+cdk_sk_protect( cdk_pkt_seckey_t sk, const char * pass )
 {
-    GCRY_CIPHER_HD hd;
-    GCRY_MD_HD md;
-    CDK_MPI a;
-    CDK_DEK dek;
-    CDK_S2K_HD s2k;
+    cdk_cipher_hd_t hd;
+    cdk_md_hd_t md;
+    cdk_mpi_t a;
+    cdk_dek_t dek;
+    cdk_s2k_t s2k;
     byte * p;
     size_t enclen = 0, nskey, i;
     int rc;
@@ -780,7 +774,7 @@ cdk_sk_protect( cdkPKT_secret_key * sk, const char * pass )
     rc = cdk_s2k_new( &s2k, 3, CDK_MD_SHA1, NULL );
     if( rc )
         return rc;
-    rc = cdk_dek_from_passphrase( &dek, GCRY_CIPHER_3DES, s2k, 2, pass );
+    rc = cdk_dek_from_passphrase( &dek, CDK_CIPHER_3DES, s2k, 2, pass );
     if( rc )
         return rc;
     
@@ -803,33 +797,34 @@ cdk_sk_protect( cdkPKT_secret_key * sk, const char * pass )
     enclen += 20;
     sk->enclen = enclen;
     sk->protect.s2k = s2k;
-    sk->protect.algo = GCRY_CIPHER_3DES;
-    sk->protect.ivlen = gcry_cipher_get_algo_blklen( sk->protect.algo  );
+    sk->protect.algo = CDK_CIPHER_3DES;
+    sk->protect.ivlen = cdk_cipher_get_algo_blklen( sk->protect.algo  );
     gcry_randomize( sk->protect.iv, sk->protect.ivlen, GCRY_STRONG_RANDOM  );
-    hd = cipher_open( sk->protect.algo, dek->key, dek->keylen,
-                      sk->protect.iv, sk->protect.ivlen );
+    hd = cdk_cipher_open( sk->protect.algo, 1,
+                          dek->key, dek->keylen,
+                          sk->protect.iv, sk->protect.ivlen );
     if( !hd ) {
         cdk_free( p );
         return CDK_Gcry_Error;
     }
 
-    md = gcry_md_open( CDK_MD_SHA1, GCRY_CIPHER_SECURE  );
+    md = cdk_md_open( CDK_MD_SHA1, GCRY_CIPHER_SECURE  );
     if( !md ) {
-        gcry_cipher_close( hd  );
+        cdk_cipher_close( hd  );
         cdk_free( p  );
         return CDK_Gcry_Error;
     }
     sk->protect.sha1chk = 1;
     sk->is_protected = 1;
     sk->csum = 0;
-    gcry_md_write( md, p, enclen - 20 );
-    gcry_md_final( md );
-    memcpy( p + enclen - 20, gcry_md_read( md, 0 ), 20 );
-    gcry_md_close( md );
-    gcry_cipher_encrypt( hd, p, enclen, NULL, 0 );
-    gcry_cipher_close( hd );
+    cdk_md_write( md, p, enclen - 20 );
+    cdk_md_final( md );
+    memcpy( p + enclen - 20, cdk_md_read( md, 0 ), 20 );
+    cdk_md_close( md );
+    rc = cdk_cipher_encrypt( hd, p, p, enclen );
+    cdk_cipher_close( hd );
     cdk_dek_free( dek );
-    return 0;
+    return rc;
 }
 
 
@@ -840,8 +835,8 @@ cdk_sk_protect( cdkPKT_secret_key * sk, const char * pass )
  *
  * Create a new public key from a secret key.
  **/
-int
-cdk_pk_from_secret_key( cdkPKT_secret_key * sk, cdkPKT_public_key ** ret_pk )
+cdk_error_t
+cdk_pk_from_secret_key( cdk_pkt_seckey_t sk, cdk_pkt_pubkey_t* ret_pk )
 {
     if( !sk )
         return CDK_Inv_Value;
@@ -850,7 +845,7 @@ cdk_pk_from_secret_key( cdkPKT_secret_key * sk, cdkPKT_public_key ** ret_pk )
 
 
 int
-cdk_pk_revoke_import( CDK_KEYDB_HD db, const char * revcert )
+cdk_pk_revoke_import( cdk_keydb_hd_t db, const char * revcert )
 {
     /* due to the fact the keydb code can't insert or modify packets
        it is not possible to handle this step yet */
@@ -859,12 +854,12 @@ cdk_pk_revoke_import( CDK_KEYDB_HD db, const char * revcert )
 
 
 int
-cdk_pk_revoke_create( cdkPKT_secret_key * sk, int code, const char * inf,
+cdk_pk_revoke_create( cdk_pkt_seckey_t sk, int code, const char * inf,
                       char ** ret_revcert )
 {
-    GCRY_MD_HD md;
-    cdkPKT_signature * sig;
-    CDK_SUBPKT node;
+    cdk_md_hd_t md;
+    cdk_subpkt_t node;
+    cdk_pkt_signature_t sig;
     char * p = NULL, * dat;
     int n;
     
@@ -894,24 +889,24 @@ cdk_pk_revoke_create( cdkPKT_secret_key * sk, int code, const char * inf,
 
     node = cdk_subpkt_new( n );
     if( node ) {
-        _cdk_subpkt_init( node, SIGSUBPKT_REVOC_REASON, dat, n );
-        _cdk_subpkt_add( sig->hashed, node );
+        cdk_subpkt_init( node, CDK_SIGSUBPKT_REVOC_REASON, dat, n );
+        cdk_subpkt_add( sig->hashed, node );
     }
     cdk_free( dat );
 
-    md = gcry_md_open( GCRY_MD_SHA1, 0 );
+    md = cdk_md_open( CDK_MD_SHA1, 0 );
     if( !md ) {
         _cdk_free_signature( sig );
         return CDK_Gcry_Error;
     }
-    _cdk_hash_pubkey( sk->pk, md, 1 );
+    _cdk_hash_pubkey( sk->pk, md, 0 );
     _cdk_free_signature( sig );
     return 0;
 }
 
 
 int
-_cdk_sk_get_csum( cdkPKT_secret_key * sk )
+_cdk_sk_get_csum( cdk_pkt_seckey_t sk )
 {
     u16 csum = 0, i;
 
@@ -924,9 +919,9 @@ _cdk_sk_get_csum( cdkPKT_secret_key * sk )
 
 
 int
-cdk_pk_get_fingerprint( cdkPKT_public_key * pk, byte * fpr )
+cdk_pk_get_fingerprint( cdk_pkt_pubkey_t pk, byte * fpr )
 {
-    GCRY_MD_HD hd;
+    cdk_md_hd_t hd;
     int md_algo;
     int dlen = 0;
 
@@ -934,17 +929,17 @@ cdk_pk_get_fingerprint( cdkPKT_public_key * pk, byte * fpr )
         return CDK_Inv_Value;
 
     if( pk->version < 4 && is_RSA( pk->pubkey_algo ) )
-        md_algo = GCRY_MD_MD5; /* special */
+        md_algo = CDK_MD_MD5; /* special */
     else
         md_algo = pk->version < 4 ? CDK_MD_RMD160 : CDK_MD_SHA1;
-    dlen = gcry_md_get_algo_dlen( md_algo );
-    hd = gcry_md_open( md_algo, 0 );
+    dlen = cdk_md_get_algo_dlen( md_algo );
+    hd = cdk_md_open( md_algo, 0 );
     if( !hd )
         return CDK_Gcry_Error;
     _cdk_hash_pubkey( pk, hd, 1 );
-    gcry_md_final( hd );
-    memcpy( fpr, gcry_md_read( hd, md_algo ), dlen );
-    gcry_md_close( hd );
+    cdk_md_final( hd );
+    memcpy( fpr, cdk_md_read( hd, md_algo ), dlen );
+    cdk_md_close( hd );
     if( dlen == 16 )
         memset( fpr + 16, 0, 4 );
     return 0;
@@ -972,7 +967,7 @@ cdk_pk_fingerprint_get_keyid( const byte * fpr, size_t fprlen, u32 * keyid )
 
 
 u32
-cdk_pk_get_keyid( cdkPKT_public_key * pk, u32 * keyid )
+cdk_pk_get_keyid( cdk_pkt_pubkey_t pk, u32 * keyid )
 {
     u32 lowbits = 0;
     byte buf[24];
@@ -1000,21 +995,21 @@ cdk_pk_get_keyid( cdkPKT_public_key * pk, u32 * keyid )
 
 
 u32
-cdk_sk_get_keyid( cdkPKT_secret_key * sk, u32 * keyid )
+cdk_sk_get_keyid( cdk_pkt_seckey_t sk, u32 * keyid )
 {
-    u32 kid = 0;
+    u32 lowbits = 0;
     
     if( sk && sk->pk ) {
-        kid = cdk_pk_get_keyid( sk->pk, keyid );
+        lowbits = cdk_pk_get_keyid( sk->pk, keyid );
         sk->keyid[0] = sk->pk->keyid[0];
         sk->keyid[1] = sk->pk->keyid[1];
     }
-    return kid;
+    return lowbits;
 }
 
 
 u32
-cdk_sig_get_keyid( cdkPKT_signature * sig, u32 * keyid )
+cdk_sig_get_keyid( cdk_pkt_signature_t sig, u32 * keyid )
 {
     u32 lowbits = sig ? sig->keyid[1] : 0;
   
@@ -1027,7 +1022,7 @@ cdk_sig_get_keyid( cdkPKT_signature * sig, u32 * keyid )
 
 
 u32
-_cdk_pkt_get_keyid( CDK_PACKET * pkt, u32 * keyid )
+_cdk_pkt_get_keyid( cdk_packet_t pkt, u32 * keyid )
 {
     u32 lowbits;
 
@@ -1057,7 +1052,7 @@ _cdk_pkt_get_keyid( CDK_PACKET * pkt, u32 * keyid )
 
 
 int
-_cdk_pkt_get_fingerprint( CDK_PACKET * pkt, byte * fpr )
+_cdk_pkt_get_fingerprint( cdk_packet_t pkt, byte * fpr )
 {
     if( !pkt || !fpr )
         return CDK_Inv_Value;
@@ -1076,5 +1071,3 @@ _cdk_pkt_get_fingerprint( CDK_PACKET * pkt, byte * fpr )
     }
     return 0;
 }
-
-
